package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import org.apache.commons.lang3.math.NumberUtils;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.bpcl.facades.payment.refund.RefundMsdData;
import java.util.Map;
import java.util.HashMap;
import com.bpcl.lpg.core.utill.BillDeskMsgUtils;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Date;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import com.bpcl.core.custom.exception.BpclInvalidPaymentResponseException;
import com.bpcl.core.custom.exception.BpclUnAuthorizedPaymentResponse;
import org.apache.commons.lang3.StringUtils;
import com.bpcl.lpg.core.enums.LpgPaymentStatusEnum;
import com.bpcl.lpg.core.enums.RefillBookingStatusEnum;
import de.hybris.platform.tx.Transaction;
import java.text.MessageFormat;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import java.text.SimpleDateFormat;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import com.bpcl.lpg.core.payment.billdesk.LPGRefundService;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import com.bpcl.lpg.core.pojo.BillDeskResponseMsgData;
import java.util.List;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentService;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentStatusCheckService;
import com.bpcl.lpg.core.payment.billdesk.dao.LPGPaymentInfoDao;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.core.model.BpclCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class PaymentReconciliationJob extends AbstractJobPerformable<BpclCronjobModel>
{
    private static final Logger LOG;
    private ConfigurationService configurationService;
    private LPGPaymentInfoDao lpgPaymentInfoDao;
    private LPGPaymentStatusCheckService lpgPaymentStatusCheckService;
    private LPGPaymentService lpgPaymentService;
    private Converter<List<String>, BillDeskResponseMsgData> queryPaymentResponseDataConverter;
    private KeyGenerator paymentTransactionIdGenerator;
    private LPGRefundService lpgRefundService;
    private NotificationEngineService notificationEngineService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final BpclCronjobModel cronjob) {
        final Date currentDate = cronjob.getStartTime();
        PaymentReconciliationJob.LOG.info("Cronjob started processing on : {}", currentDate);
        try {
            final Date startDate = this.populateStartDate(cronjob);
            PaymentReconciliationJob.LOG.info("Date ranges starts from : {}", startDate);
            final Date endDate = this.populateEndDate(cronjob);
            PaymentReconciliationJob.LOG.info("Filter End Date : {}", endDate);
            final List<LpgPaymentInfoModel> transactionList = this.prepareTransactionList(currentDate, startDate, endDate);
            PaymentReconciliationJob.LOG.info("Number of Txns to be processed: {}", (Object)transactionList.size());
            final SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
            for (final LpgPaymentInfoModel transaction : transactionList) {
                PaymentReconciliationJob.LOG.info(MessageFormat.format("Processing transaction {0}", transaction.getTransactionID()));
                boolean isPaymentReconciled = false;
                final Transaction tx = Transaction.current();
                tx.begin();
                try {
                    this.modelService.refresh((Object)transaction);
                    if (null != transaction.getRefillBookingStatus() && transaction.getRefillBookingStatus().equals((Object)RefillBookingStatusEnum.SUCCESSFUL)) {
                        PaymentReconciliationJob.LOG.info(MessageFormat.format("Transaction {0} has Refill booking status as successful,No refund required", transaction.getTransactionID()));
                        transaction.setTransactionStatus(LpgPaymentStatusEnum.SUCCESSFUL);
                        this.modelService.save((Object)transaction);
                        continue;
                    }
                    if (null != transaction.getRefillBookingStatus() && null != transaction.getTransactionStatus() && !StringUtils.equalsIgnoreCase((CharSequence)transaction.getRefillBookingStatus().getCode(), (CharSequence)RefillBookingStatusEnum.SUCCESSFUL.getCode()) && !StringUtils.equalsIgnoreCase((CharSequence)transaction.getTransactionStatus().getCode(), (CharSequence)LpgPaymentStatusEnum.SUCCESSFUL.getCode())) {
                        break;
                    }
                    this.modelService.lock(transaction.getPk());
                    this.performReconciliation(transaction.getTransactionID(), transaction, format);
                    isPaymentReconciled = true;
                    continue;
                }
                catch (final BpclUnAuthorizedPaymentResponse upEx) {
                    PaymentReconciliationJob.LOG.error(MessageFormat.format("Exception occured {0} for Txn id {1}", upEx.getCause(), transaction.getTransactionID()));
                }
                catch (final BpclInvalidPaymentResponseException ipEx) {
                    PaymentReconciliationJob.LOG.error(MessageFormat.format("Exception occured {0} for Txn id {1}", ipEx.getCause(), transaction.getTransactionID()));
                }
                catch (final Exception txException) {
                    PaymentReconciliationJob.LOG.error("exception on the locked transaction ", txException);
                }
                finally {
                    if (isPaymentReconciled) {
                        tx.commit();
                    }
                    else {
                        tx.rollback();
                    }
                }
            }
            return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
        }
        catch (final Exception ex) {
            PaymentReconciliationJob.LOG.error(ex.getMessage(), ex.getCause());
            final String body = MessageFormat.format("Payment reconciliation cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Payment Reconciliation Job Alert", body, this.configurationService.getConfiguration().getString("lpg.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
    }
    
    private Date populateStartDate(final BpclCronjobModel cronjob) {
        Date startDate = cronjob.getFilterStartDate();
        if (startDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.add(10, -this.configurationService.getConfiguration().getInt("lpg.payment.reconciliation.data.start.period"));
            startDate = cal.getTime();
        }
        return startDate;
    }
    
    private Date populateEndDate(final BpclCronjobModel cronjob) {
        Date endDate = cronjob.getFilterEndDate();
        if (endDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.add(10, -this.configurationService.getConfiguration().getInt("lpg.payment.reconciliation.data.end.period"));
            endDate = cal.getTime();
        }
        return endDate;
    }
    
    private void performReconciliation(final String txnId, final LpgPaymentInfoModel transaction, final SimpleDateFormat format) throws BpclUnAuthorizedPaymentResponse, BpclInvalidPaymentResponseException, JsonProcessingException {
        final String msg = this.lpgPaymentStatusCheckService.getPaymentStatusFromBillDeskByTxnID(txnId);
        if (StringUtils.isNotBlank((CharSequence)msg)) {
            final boolean authorizedResponse = this.lpgPaymentService.isAuthorizedResponse(msg);
            if (!authorizedResponse) {
                PaymentReconciliationJob.LOG.error("UnAuthorized response from the provider");
                throw new BpclUnAuthorizedPaymentResponse("UnAuthorized response from the provider");
            }
            final List<String> messageList = BillDeskMsgUtils.convertMsgToAttributes(msg, 33);
            final BillDeskResponseMsgData responseMsgData = (BillDeskResponseMsgData)this.queryPaymentResponseDataConverter.convert((Object)messageList);
            if (StringUtils.equalsIgnoreCase((CharSequence)responseMsgData.getAuthStatus(), (CharSequence)"0300")) {
                PaymentReconciliationJob.LOG.info("Payment is success for txn {}", txnId);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
                if (this.isValidRefundStatus(responseMsgData.getRefundStatus())) {
                    PaymentReconciliationJob.LOG.debug(MessageFormat.format("Refund already initiated for txn id {0} ", txnId));
                    transaction.setTransactionStatus(LpgPaymentStatusEnum.REFUND_SUCCESSFUL);
                    transaction.setIsRefunded(Boolean.valueOf(true));
                }
                else {
                    PaymentReconciliationJob.LOG.info("Initiating refund for txn {}", txnId);
                    final RefundMsdData refundMsdData = this.getRefundData(responseMsgData);
                    this.lpgRefundService.processRefund(refundMsdData, transaction);
                    this.modelService.refresh((Object)transaction);
                    if (StringUtils.isNotEmpty((CharSequence)transaction.getRefundProcessStatus()) && !StringUtils.equals((CharSequence)"NA", (CharSequence)transaction.getRefundProcessStatus()) && StringUtils.equals((CharSequence)"Y", (CharSequence)transaction.getRefundProcessStatus()) && StringUtils.isNotEmpty((CharSequence)transaction.getBilldeskRefundID()) && !StringUtils.equals((CharSequence)"NA", (CharSequence)transaction.getBilldeskRefundID())) {
                        final String content = MessageFormat.format(this.configurationService.getConfiguration().getString("lpg.customer.refund.notification"), String.valueOf(transaction.getRefundAmount()), transaction.getBankReferenceNo(), format.format(new Date()));
                        final String templateId = this.configurationService.getConfiguration().getString("lpg.ym.v2.cashmemo.sms.templateId");
                        final String mobileNumber = transaction.getAccount().getBpclCustomer().getUid();
                        final Map<String, String> params = new HashMap<String, String>();
                        params.put("1", String.valueOf(transaction.getRefundAmount()));
                        params.put("2", transaction.getBankReferenceNo());
                        params.put("3", format.format(new Date()));
                        this.lpgV2NotificationEngineService.sendSMS(templateId, mobileNumber, (Map)params);
                    }
                }
            }
            else if (StringUtils.equalsIgnoreCase((CharSequence)responseMsgData.getAuthStatus(), (CharSequence)"0001")) {
                PaymentReconciliationJob.LOG.info("changing status to Failed for txn {}", txnId);
                transaction.setTransactionStatus(LpgPaymentStatusEnum.FAILED);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
            }
            else if (StringUtils.equalsIgnoreCase((CharSequence)responseMsgData.getAuthStatus(), (CharSequence)"NA")) {
                PaymentReconciliationJob.LOG.info("changing status to Error for txn {}", txnId);
                transaction.setTransactionStatus(LpgPaymentStatusEnum.ERROR);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
            }
            this.modelService.save((Object)transaction);
        }
    }
    
    private String getStatus(final String authStatus) {
        return StringUtils.equalsIgnoreCase((CharSequence)authStatus, (CharSequence)"0002") ? "IN_PROGRESS" : "FAILED";
    }
    
    private boolean isValidRefundStatus(final String refundStatus) {
        return StringUtils.equalsIgnoreCase((CharSequence)refundStatus, (CharSequence)"0699") || StringUtils.equalsIgnoreCase((CharSequence)refundStatus, (CharSequence)"0799") || StringUtils.equalsIgnoreCase((CharSequence)refundStatus, (CharSequence)"0899");
    }
    
    private RefundMsdData getRefundData(final BillDeskResponseMsgData responseMsgData) {
        final RefundMsdData refundMsdData = new RefundMsdData();
        refundMsdData.setBilldeskTxnID(responseMsgData.getTxnReferenceNo());
        refundMsdData.setRefundRefNo((String)this.paymentTransactionIdGenerator.generate());
        refundMsdData.setTxnAmount(Double.valueOf(responseMsgData.getTxnAmount()).toString());
        refundMsdData.setTxnDate(responseMsgData.getTxnDate());
        refundMsdData.setTransactionID(responseMsgData.getCustomerID());
        return refundMsdData;
    }
    
    private List<LpgPaymentInfoModel> prepareTransactionList(final Date currentTime, final Date startDate, final Date endDate) {
        final int validationWindow = this.configurationService.getConfiguration().getInt("lpg.payment.reconciliation.window");
        final List<LpgPaymentInfoModel> transactionList = this.lpgPaymentInfoDao.getCurrentTxnDataFromTimeStamp(startDate, endDate, LpgPaymentStatusEnum.IN_PROGRESS);
        return transactionList.stream().filter(data -> this.isExpiredWindowTime(currentTime, data.getInitTimeStamp(), validationWindow, data.getTransactionID())).collect((Collector<? super Object, ?, List<LpgPaymentInfoModel>>)Collectors.toList());
    }
    
    private boolean isExpiredWindowTime(final Date currentTime, final Date initTimeStamp, final int validationWindow, final String transactionID) {
        if (initTimeStamp == null) {
            PaymentReconciliationJob.LOG.error(MessageFormat.format("Transaction {0} has no initiation time", transactionID));
            return Boolean.FALSE;
        }
        return NumberUtils.compare(currentTime.getTime() - initTimeStamp.getTime(), TimeUnit.MINUTES.toMillis(validationWindow)) > 0;
    }
    
    public boolean isAbortable() {
        return true;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setLpgPaymentInfoDao(final LPGPaymentInfoDao lpgPaymentInfoDao) {
        this.lpgPaymentInfoDao = lpgPaymentInfoDao;
    }
    
    public void setLpgPaymentStatusCheckService(final LPGPaymentStatusCheckService lpgPaymentStatusCheckService) {
        this.lpgPaymentStatusCheckService = lpgPaymentStatusCheckService;
    }
    
    public void setLpgPaymentService(final LPGPaymentService lpgPaymentService) {
        this.lpgPaymentService = lpgPaymentService;
    }
    
    public void setQueryPaymentResponseDataConverter(final Converter<List<String>, BillDeskResponseMsgData> queryPaymentResponseDataConverter) {
        this.queryPaymentResponseDataConverter = queryPaymentResponseDataConverter;
    }
    
    public void setPaymentTransactionIdGenerator(final KeyGenerator paymentTransactionIdGenerator) {
        this.paymentTransactionIdGenerator = paymentTransactionIdGenerator;
    }
    
    public void setLpgRefundService(final LPGRefundService lpgRefundService) {
        this.lpgRefundService = lpgRefundService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(PaymentReconciliationJob.class);
    }
}
